@@ -4,11 +4,13 @@ from __future__ import absolute_import
 import json
 import requests
 
+from .network import CFnetwork
 from .logging_helper import CFlogger
-from .utils import user_agent, sanitize_secrets
+from .utils import user_agent, build_curl
 from .read_configs import read_configs
 from .api_v4 import api_v4
 from .api_extras import api_extras
+from .api_decode_from_web import api_decode_from_web
 from .exceptions import CloudFlareError, CloudFlareAPIError, CloudFlareInternalError
 
 BASE_URL = 'https://api.cloudflare.com/client/v4'
@@ -19,20 +21,35 @@ class CloudFlare(object):
     class _v4base(object):
         """ Cloudflare v4 API"""
 
-        def __init__(self, email, token, certtoken, base_url, debug, raw, use_sessions):
+        def __init__(self, config):
             """ Cloudflare v4 API"""
 
-            self.email = email
-            self.token = token
-            self.certtoken = certtoken
-            self.base_url = base_url
-            self.raw = raw
-            self.use_sessions = use_sessions
-            self.session = None
+            self.config = config
+            if 'email' in config:
+                self.email = config['email']
+            else:
+                self.email = None
+            if 'token' in config:
+                self.token = config['token']
+            else:
+                self.token = None
+            if 'certtoken' in config:
+                self.certtoken = config['certtoken']
+            else:
+                self.certtoken = None
+            if 'base_url' in config:
+                self.base_url = config['base_url']
+            else:
+                # We must have a base_url value
+                self.base_url = BASE_URL
+            self.raw = config['raw']
+            self.use_sessions = config['use_sessions']
+            self.profile = config['profile']
+            self.network = CFnetwork(use_sessions=self.use_sessions)
             self.user_agent = user_agent()
 
-            if debug:
-                self.logger = CFlogger(debug).getLogger()
+            if 'debug' in config and config['debug']:
+                self.logger = CFlogger(config['debug']).getLogger()
             else:
                 self.logger = None
 
@@ -41,28 +58,64 @@ class CloudFlare(object):
                               params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
-            headers = {
-                'User-Agent': self.user_agent,
-                'Content-Type': 'application/json'
-            }
+            headers = {}
+            self._AddHeaders(headers)
             return self._call(method, headers, parts,
                               identifier1, identifier2, identifier3,
                               params, data, files)
 
+        def _AddHeaders(self, headers):
+            """ Add default headers """
+            headers['User-Agent'] = self.user_agent
+            headers['Content-Type'] = 'application/json'
+
+        def _AddAuthHeaders(self, headers, method):
+            """ Add authentication headers """
+
+            v = 'email' + '.' + method.lower()
+            if v in self.config:
+                email = self.config[v] # use specific value for this method
+            else:
+                email = self.email     # use generic value for all methods
+
+            v = 'token' + '.' + method.lower()
+            if v in self.config:
+                token = self.config[v] # use specific value for this method
+            else:
+                token = self.token     # use generic value for all methods
+
+            if email is None and token is None:
+                raise CloudFlareAPIError(0, 'no email and no token defined')
+            if token is None:
+                raise CloudFlareAPIError(0, 'no token defined')
+            if email is None:
+                headers['Authorization'] = 'Bearer %s' % (token)
+            else:
+                headers['X-Auth-Email'] = email
+                headers['X-Auth-Key'] = token
+
+        def _AddCerttokenHeaders(self, headers, method):
+            """ Add authentication headers """
+
+            v = 'certtoken' + '.' + method.lower()
+            if v in self.config:
+                certtoken = self.config[v] # use specific value for this method
+            else:
+                certtoken = self.certtoken # use generic value for all methods
+
+            if certtoken is None:
+                raise CloudFlareAPIError(0, 'no cert token defined')
+            headers['X-Auth-User-Service-Key'] = certtoken
+
         def call_with_auth(self, method, parts,
                            identifier1=None, identifier2=None, identifier3=None,
                            params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
-            if self.email is '' or self.token is '':
-                raise CloudFlareAPIError(0, 'no email and/or token defined')
-            headers = {
-                'User-Agent': self.user_agent,
-                'X-Auth-Email': self.email,
-                'X-Auth-Key': self.token,
-                'Content-Type': 'application/json'
-            }
-            if type(data) == str:
+            headers = {}
+            self._AddHeaders(headers)
+            self._AddAuthHeaders(headers, method)
+            if isinstance(data, str):
                 # passing javascript vs JSON
                 headers['Content-Type'] = 'application/javascript'
             if files:
@@ -79,15 +132,10 @@ class CloudFlare(object):
                                      params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
-            if self.email is '' or self.token is '':
-                raise CloudFlareAPIError(0, 'no email and/or token defined')
-            headers = {
-                'User-Agent': self.user_agent,
-                'X-Auth-Email': self.email,
-                'X-Auth-Key': self.token,
-                'Content-Type': 'application/json'
-            }
-            if type(data) == str:
+            headers = {}
+            self._AddHeaders(headers)
+            self._AddAuthHeaders(headers, method)
+            if isinstance(data, str):
                 # passing javascript vs JSON
                 headers['Content-Type'] = 'application/javascript'
             if files:
@@ -104,36 +152,18 @@ class CloudFlare(object):
                                params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
-            if self.certtoken is '' or self.certtoken is None:
-                raise CloudFlareAPIError(0, 'no cert token defined')
-            headers = {
-                'User-Agent': self.user_agent,
-                'X-Auth-User-Service-Key': self.certtoken,
-                'Content-Type': 'application/json'
-            }
+            headers = {}
+            self._AddHeaders(headers)
+            self._AddCerttokenHeaders(headers, method)
             return self._call(method, headers, parts,
                               identifier1, identifier2, identifier3,
                               params, data, files)
 
-        def _network(self, method, headers, parts,
+        def _call_network(self, method, headers, parts,
                      identifier1=None, identifier2=None, identifier3=None,
                      params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
-            if self.logger:
-                self.logger.debug('Call: %s,%s,%s,%s,%s,%s',
-                                  str(parts[0]),
-                                  str(identifier1),
-                                  str(parts[1]),
-                                  str(identifier2),
-                                  str(parts[2]),
-                                  str(identifier3))
-                self.logger.debug('Call: optional params and data %s %s',
-                                  str(params),
-                                  str(data))
-                if files:
-                    self.logger.debug('Call: upload file %r', files)
-
             if (method is None) or (parts[0] is None):
                 # should never happen
                 raise CloudFlareInternalError(0, 'You must specify a method and endpoint')
@@ -166,85 +196,16 @@ class CloudFlare(object):
                 url += '/' + identifier3
 
             if self.logger:
-                self.logger.debug('Call: method and url %s %s', str(method), str(url))
-                self.logger.debug('Call: headers %s', str(sanitize_secrets(headers)))
-
-            method = method.upper()
-
-            if self.logger:
-                self.logger.debug('Call: doit!')
-
-            if self.use_sessions:
-                if self.session is None:
-                    self.session = requests.Session()
-            else:
-                self.session = requests
+                msg = build_curl(method, url, headers, params, data, files)
+                self.logger.debug('Call: emulated curl command ...\n' + msg)
 
             try:
-                if method == 'GET':
-                    response = self.session.get(url,
-                                                headers=headers,
-                                                params=params,
-                                                data=data)
-                elif method == 'POST':
-                    if type(data) == str:
-                        response = self.session.post(url,
-                                                     headers=headers,
-                                                     params=params,
-                                                     data=data,
-                                                     files=files)
-                    else:
-                        response = self.session.post(url,
-                                                     headers=headers,
-                                                     params=params,
-                                                     json=data,
-                                                     files=files)
-                elif method == 'PUT':
-                    if type(data) == str:
-                        response = self.session.put(url,
-                                                    headers=headers,
-                                                    params=params,
-                                                    data=data)
-                    else:
-                        response = self.session.put(url,
-                                                    headers=headers,
-                                                    params=params,
-                                                    json=data)
-                elif method == 'DELETE':
-                    if type(data) == str:
-                        response = self.session.delete(url,
-                                                       headers=headers,
-                                                       params=params,
-                                                       data=data)
-                    else:
-                        response = self.session.delete(url,
-                                                       headers=headers,
-                                                       params=params,
-                                                       json=data)
-                elif method == 'PATCH':
-                    if type(data) == str:
-                        response = self.session.request('PATCH', url,
-                                                        headers=headers,
-                                                        params=params,
-                                                        data=data)
-                    else:
-                        response = self.session.request('PATCH', url,
-                                                        headers=headers,
-                                                        params=params,
-                                                        json=data)
-                else:
-                    # should never happen
-                    raise CloudFlareAPIError(0, 'method not supported')
-                if self.logger:
-                    self.logger.debug('Call: done!')
+                response = self.network(method, url, headers, params, data, files)
             except Exception as e:
                 if self.logger:
-                    self.logger.debug('Call: exception!')
+                    self.logger.debug('Call: exception! "%s"' % (e))
                 raise CloudFlareAPIError(0, 'connection failed.')
 
-            if self.logger:
-                self.logger.debug('Response: url %s', response.url)
-
             # Create response_{type|code|data}
             try:
                 response_type = response.headers['Content-Type']
@@ -257,7 +218,7 @@ class CloudFlare(object):
                 response_type = 'application/octet-stream'
             response_code = response.status_code
             response_data = response.content
-            if type(response_data) != str:
+            if not isinstance(response_data, (str, bytes, bytearray)):
                 response_data = response_data.decode("utf-8")
 
             if self.logger:
@@ -311,13 +272,13 @@ class CloudFlare(object):
                  params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
-            [response_type, response_code, response_data] = self._network(method,
-                                                                          headers, parts,
-                                                                          identifier1,
-                                                                          identifier2,
-                                                                          identifier3,
-                                                                          params, data, files)
-
+            [response_type, response_code, response_data] = self._call_network(method,
+                                                                               headers, parts,
+                                                                               identifier1,
+                                                                               identifier2,
+                                                                               identifier3,
+                                                                               params, data, files)
+     
             if response_type == 'application/json':
                 # API says it's JSON; so it better be parsable as JSON
                 # NDJSON is returned by Enterprise Log Share i.e. /zones/:id/logs/received
@@ -325,6 +286,9 @@ class CloudFlare(object):
                     response_data = response_data.decode('utf-8')
                 try:
                     response_data = json.loads(response_data)
+                    if not isinstance(response_data, (dict)):
+                        response_data = {'success': True,
+                                         'result': response_data}
                 except ValueError:
                     if response_data == '':
                         # This should really be 'null' but it isn't. Even then, it's wrong!
@@ -358,23 +322,64 @@ class CloudFlare(object):
                     # 3xx & 4xx errors - we should report that somehow - but not quite yet
                     # response_data['code'] = response_code
                     pass
+            elif response_type == 'application/octet-stream' and isinstance(response_data, (int, float)):
+                # It's binary data
+                if response_code == requests.codes.ok:
+                    # 200 ok
+                    response_data = {'success': True,
+                                     'result': response_data}
+                else:
+                    # 3xx & 4xx errors
+                    response_data = {'success': False,
+                                     'code': response_code,
+                                     'result': response_data}
+            elif response_type == 'application/octet-stream' and isinstance(response_data, (bytes, bytearray)):
+                # API says it's text; but maybe it's actually JSON? - should be fixed in API
+                if hasattr(response_data, 'decode'):
+                    response_data = response_data.decode('utf-8')
+                try:
+                    response_data = json.loads(response_data)
+                    if not isinstance(response_data, (dict)) or 'success' not in response_data:
+                        if response_code == requests.codes.ok:
+                            # 200 ok
+                            response_data = {'success': True,
+                                             'result': response_data}
+                        else:
+                            # 3xx & 4xx errors
+                            response_data = {'success': False,
+                                             'code': response_code,
+                                             'result': response_data}
+                except ValueError:
+                    # So it wasn't JSON - moving on as if it's text!
+                    # A single value is returned (vs an array or object)
+                    if response_code == requests.codes.ok:
+                        # 200 ok
+                        response_data = {'success': True, 'result': response_data}
+                    else:
+                        # 3xx & 4xx errors
+                        response_data = {'success': False,
+                                         'code': response_code,
+                                         'result': response_data}
             elif response_type == 'text/plain' or response_type == 'application/octet-stream':
                 # API says it's text; but maybe it's actually JSON? - should be fixed in API
                 if hasattr(response_data, 'decode'):
                     response_data = response_data.decode('utf-8')
                 try:
                     response_data = json.loads(response_data)
+                    if not isinstance(response_data, (dict)):
+                        response_data = {'success': True,
+                                         'result': response_data}
                 except ValueError:
                     # So it wasn't JSON - moving on as if it's text!
                     # A single value is returned (vs an array or object)
                     if response_code == requests.codes.ok:
                         # 200 ok
-                        response_data = {'success': True, 'result': str(response_data)}
+                        response_data = {'success': True, 'result': response_data}
                     else:
                         # 3xx & 4xx errors
                         response_data = {'success': False,
                                          'code': response_code,
-                                         'result': str(response_data)}
+                                         'result': response_data}
             elif response_type == 'text/javascript' or response_type == 'application/javascript':
                 # used by Cloudflare workers
                 if response_code == requests.codes.ok:
@@ -426,9 +431,29 @@ class CloudFlare(object):
             # Sanatize the returned results - just in case API is messed up
             if 'success' not in response_data:
                 if 'errors' in response_data:
-                    if self.logger:
-                        self.logger.debug('Response: assuming success = "False"')
-                    response_data['success'] = False
+                    if response_data['errors'] == None:
+                        # Only happens on /graphql call
+                        if self.logger:
+                            self.logger.debug('Response: assuming success = "True"')
+                        response_data['success'] = True
+                    else:
+                        if self.logger:
+                            self.logger.debug('Response: assuming success = "False"')
+                        # The following only happens on /graphql call
+                        try:
+                            message = response_data['errors'][0]['message']
+                        except:
+                            message = ''
+                        try:
+                            location = str(response_data['errors'][0]['location'])
+                        except:
+                            location = ''
+                        try:
+                            path = '>'.join(response_data['errors'][0]['path'])
+                        except:
+                            path = ''
+                        response_data['errors'] = [{'code': 99999, 'message': message + ' - ' + location + ' - ' + path}]
+                        response_data['success'] = False
                 else:
                     if 'result' not in response_data:
                         # Only happens on /certificates call
@@ -445,16 +470,22 @@ class CloudFlare(object):
                         response_data['success'] = True
 
             if response_data['success'] is False:
-                errors = response_data['errors'][0]
-                code = errors['code']
+                if 'errors' in response_data:
+                    errors = response_data['errors'][0]
+                else:
+                    errors = {}
+                if 'code' in errors:
+                    code = errors['code']
+                else:
+                    code = 99998
                 if 'message' in errors:
                     message = errors['message']
                 elif 'error' in errors:
                     message = errors['error']
                 else:
                     message = ''
-                if 'messages' in response_data:
-                    errors['error_chain'] = response_data['messages']
+                ##if 'messages' in response_data:
+                ##    errors['error_chain'] = response_data['messages']
                 if 'error_chain' in errors:
                     error_chain = errors['error_chain']
                     for error in error_chain:
@@ -470,19 +501,25 @@ class CloudFlare(object):
                         self.logger.debug('Response: error %d %s', code, message)
                     raise CloudFlareAPIError(code, message)
 
-            if self.logger:
-                self.logger.debug('Response: %s', response_data['result'])
             if self.raw:
                 result = {}
-                # theres always a result value
-                result['result'] = response_data['result']
+                # theres always a result value - unless it's a graphql query
+                try:
+                    result['result'] = response_data['result']
+                except:
+                    result['result'] = response_data
                 # theres may not be a result_info on every call
                 if 'result_info' in response_data:
                     result['result_info'] = response_data['result_info']
                 # no need to return success, errors, or messages as they return via an exception
             else:
-                # theres always a result value
-                result = response_data['result']
+                # theres always a result value - unless it's a graphql query
+                try:
+                    result = response_data['result']
+                except:
+                    result = response_data
+            if self.logger:
+                self.logger.debug('Response: %s', result)
             return result
 
         def _call_unwrapped(self, method, headers, parts,
@@ -498,7 +535,26 @@ class CloudFlare(object):
             result = response_data
             return result
 
-    class _add_unused(object):
+        def _api_from_web(self):
+            """ Cloudflare v4 API"""
+
+            # base url isn't enough; we need less
+            url = '/'.join(self.base_url.split('/')[0:3])
+
+            try:
+                if self.logger:
+                    self.logger.debug('Call: doit!')
+                response = self.network('GET', url)
+                if self.logger:
+                    self.logger.debug('Call: done!')
+            except Exception as e:
+                if self.logger:
+                    self.logger.debug('Call: exception! "%s"' % (e))
+                raise CloudFlareAPIError(0, 'connection failed.')
+
+            return response.text
+
+    class _AddUnused(object):
         """ Cloudflare v4 API"""
 
         def __init__(self, base, p1, p2=None, p3=None):
@@ -543,7 +599,7 @@ class CloudFlare(object):
 
             raise CloudFlareAPIError(0, 'delete() call not available for this endpoint')
 
-    class _add_noauth(object):
+    class _AddNoAuth(object):
         """ Cloudflare v4 API"""
 
         def __init__(self, base, p1, p2=None, p3=None):
@@ -590,7 +646,7 @@ class CloudFlare(object):
 
             raise CloudFlareAPIError(0, 'delete() call not available for this endpoint')
 
-    class _add_with_auth(object):
+    class _AddWithAuth(object):
         """ Cloudflare v4 API"""
 
         def __init__(self, base, p1, p2=None, p3=None):
@@ -645,7 +701,7 @@ class CloudFlare(object):
                                              identifier1, identifier2, identifier3,
                                              params, data)
 
-    class _add_with_auth_unwrapped(object):
+    class _AddWithAuthUnwrapped(object):
         """ Cloudflare v4 API"""
 
         def __init__(self, base, p1, p2=None, p3=None):
@@ -700,7 +756,7 @@ class CloudFlare(object):
                                                        identifier1, identifier2, identifier3,
                                                        params, data)
 
-    class _add_with_cert_auth(object):
+    class _AddWithCertAuth(object):
         """ Cloudflare v4 API"""
 
         def __init__(self, base, p1, p2=None, p3=None):
@@ -769,27 +825,45 @@ class CloudFlare(object):
         branch = self
         for element in a[0:-1]:
             try:
-                branch = getattr(branch, element)
+                if '-' in element:
+                    branch = getattr(element, element.replace('-','_'))
+                else:
+                    branch = getattr(branch, element)
             except:
                 # should never happen
                 raise CloudFlareAPIError(0, 'api load name failed')
         name = a[-1]
 
+        try:
+            if '-' in name:
+                f = getattr(element, name.replace('-','_'))
+            else:
+                f = getattr(branch, name)
+            # already exists - don't let it overwrite
+            raise CloudFlareAPIError(0, 'api duplicate name found: %s/**%s**' % ('/'.join(a[0:-1]), name))
+        except AttributeError:
+            # this is the required behavior - i.e. it's a new node to create
+            pass
+
         if t == 'VOID':
-            f = self._add_unused(self._base, p1, p2, p3)
+            f = self._AddUnused(self._base, p1, p2, p3)
         elif t == 'OPEN':
-            f = self._add_noauth(self._base, p1, p2, p3)
+            f = self._AddNoAuth(self._base, p1, p2, p3)
         elif t == 'AUTH':
-            f = self._add_with_auth(self._base, p1, p2, p3)
+            f = self._AddWithAuth(self._base, p1, p2, p3)
         elif t == 'CERT':
-            f = self._add_with_cert_auth(self._base, p1, p2, p3)
+            f = self._AddWithCertAuth(self._base, p1, p2, p3)
         elif t == 'AUTH_UNWRAPPED':
-            f = self._add_with_auth_unwrapped(self._base, p1, p2, p3)
+            f = self._AddWithAuthUnwrapped(self._base, p1, p2, p3)
         else:
             # should never happen
             raise CloudFlareAPIError(0, 'api load type mismatch')
 
-        setattr(branch, name, f)
+        if '-' in name:
+            # dashes (vs underscores) cause issues in Python and other languages
+            setattr(branch, name.replace('-','_'), f)
+        else:
+            setattr(branch, name, f)
 
     def api_list(self, m=None, s=''):
         """recursive walk of the api tree returning a list of api calls"""
@@ -810,31 +884,54 @@ class CloudFlare(object):
                 if 'delete' in d or 'get' in d or 'patch' in d or 'post' in d or 'put' in d:
                     # only show the result if a call exists for this part
                     if '_parts' in d:
-                        w.append(s + '/' + n)
+                        # handle underscores by returning the actual API call vs the method name
+                        w.append(str(a)[1:-1].replace('/:id/','/'))
                 w = w + self.api_list(a, s + '/' + n)
         return w
 
-    def __init__(self, email=None, token=None, certtoken=None, debug=False, raw=False, use_sessions=True):
+    def api_from_web(self):
         """ Cloudflare v4 API"""
 
-        base_url = BASE_URL
+        return api_decode_from_web(self._base._api_from_web())
 
-        # class creation values override configuration values
-        [conf_email, conf_token, conf_certtoken, extras] = read_configs()
+    def __init__(self, email=None, token=None, certtoken=None, debug=False, raw=False, use_sessions=True, profile=None, base_url=None):
+        """ Cloudflare v4 API"""
 
-        if email is None:
-            email = conf_email
-        if token is None:
-            token = conf_token
-        if certtoken is None:
-            certtoken = conf_certtoken
+        try:
+            config = read_configs(profile)
+        except Exception as e:
+            raise CloudFlareAPIError(0, str(e))
 
-        self._base = self._v4base(email, token, certtoken, base_url, debug, raw, use_sessions)
+        # class creation values override all configuration values
+        if email is not None:
+            config['email'] = email
+        if token is not None:
+            config['token'] = token
+        if certtoken is not None:
+            config['certtoken'] = certtoken
+        if base_url is not None:
+            config['base_url'] = base_url
+        if debug is not None:
+            config['debug'] = debug
+        if raw is not None:
+            config['raw'] = raw
+        if use_sessions is not None:
+            config['use_sessions'] = use_sessions
+        if profile is not None:
+            config['profile'] = profile
+
+        # we do not need to handle item.call values - they pass straight thru
+
+        for x in config:
+            if config[x] == '':
+                config[x] = None
+
+        self._base = self._v4base(config)
 
         # add the API calls
         api_v4(self)
-        if extras:
-            api_extras(self, extras)
+        if 'extras' in config and config['extras']:
+            api_extras(self, config['extras'])
 
     def __call__(self):
         """ Cloudflare v4 API"""
@@ -855,13 +952,25 @@ class CloudFlare(object):
     def __str__(self):
         """ Cloudflare v4 API"""
 
-        return '["%s","%s"]' % (self._base.email, "REDACTED")
+        if self._base.email is None:
+            s = '["%s","%s"]' % (self._base.profile, 'REDACTED')
+        else:
+            s = '["%s","%s","%s"]' % (self._base.profile, self._base.email, 'REDACTED')
+        return s
 
     def __repr__(self):
         """ Cloudflare v4 API"""
 
-        return '%s,%s(%s,"%s","%s","%s",%s,"%s")' % (
-            self.__module__, type(self).__name__,
-            self._base.email, 'REDACTED', 'REDACTED',
-            self._base.base_url, self._base.raw, self._base.user_agent
-        )
+        if self._base.email is None:
+            s = '%s,%s("%s","%s","%s","%s",%s,"%s")' % (
+                self.__module__, type(self).__name__,
+                self._base.profile, 'REDACTED', 'REDACTED',
+                self._base.base_url, self._base.raw, self._base.user_agent
+            )
+        else:
+            s = '%s,%s("%s","%s","%s","%s","%s",%s,"%s")' % (
+                self.__module__, type(self).__name__,
+                self._base.profile, self._base.email, 'REDACTED', 'REDACTED',
+                self._base.base_url, self._base.raw, self._base.user_agent
+            )
+        return s
